-- DOS font tools

without warning
without trace

include machine.e     -- We need to access BIOS interrupts


procedure add_fonts( integer start, sequence char )

    -- move a sequence of fonts into memory

    object    lowMem
    sequence  inReg, outReg

    lowMem = allocate_low( length( char ) )   -- get memory
    poke( lowMem, char )         -- move description to memory

    inReg = repeat( 0, 10 )         -- initialize register list
    inReg[REG_AX] = #1100           -- new char function
    inReg[REG_BX] = 16 * 256        -- 16 bytes/char - for 256 characters
    inReg[REG_CX] = length(char)/16 -- number of characters to change
    inReg[REG_DX] = start           -- first character to change
    inReg[REG_ES] = floor(lowMem / 16)    -- segment of low_mem (low memory)
    inReg[REG_BP] = remainder(lowMem, 16) -- offset of low_mem
    outReg = dos_interrupt( #10, inReg )  -- bios call

    free_low( lowMem )  -- free the memory

end procedure




global procedure load_font_file( sequence fileName )
    
    -- load a font file into memory
    
    sequence newChar
    object text, handle
    integer startAscii, priorAscii, newAscii, accum


    -- open the file
    handle = open( fileName, "r" ) -- font file
    if handle = -1 then         
        puts( 1, "Can't open " & fileName & ".\n" )
        abort( 1 )
    end if

    -- initialize
    newChar = {}
    priorAscii = 0
    startAscii = 0

    while 1 = 1 do

        -- read until eof
        text = gets( handle ) -- get text
        if atom( text ) then  -- eof?
            exit -- quit loop
        end if

        if text[1] = ';' then
            -- ignore, comment

        elsif text[1] = '#' or text[1] = '-' then

            -- get the character code
            if text[1] = '-' then
                -- get ascii code
                newAscii = text[2]
            else
                -- convert to number
                newAscii = 0
                for i = 2 to length( text ) do
                    if text[i] < '0'            -- not a digit
                    or text[i] > '9' then       -- not a digit
                        exit  
                    end if
                    -- convert to digits
                    newAscii = ( newAscii * 10 ) + ( text[i] - '0' )
                end for
            end if

            -- if a break in the sequence, display prior
            if newAscii != priorAscii + 1 then

                -- add if there is something to add
                if length( newChar ) > 0 then
                    add_fonts( startAscii, newChar )
                end if

                -- new start of fonts
                priorAscii = newAscii
                startAscii = newAscii
                newChar = {}
            else
                -- reset prior character
                priorAscii = newAscii
            end if

        elsif text[1] = '.' or text[1] = 'x' then
            
            -- convert to binary
            accum = 0
            if text[1] = 'x' then accum = accum + 128       end if
            if text[2] = 'x' then accum = accum + 64        end if
            if text[3] = 'x' then accum = accum + 32        end if
            if text[4] = 'x' then accum = accum + 16        end if
            if text[5] = 'x' then accum = accum + 8         end if
            if text[6] = 'x' then accum = accum + 4         end if
            if text[7] = 'x' then accum = accum + 2         end if
            if text[8] = 'x' then accum = accum + 1         end if
            
            -- store in newChar
            newChar = newChar & accum

        else
            -- junk.
        end if



    end while
    
    -- close the file
    close( handle )

end procedure



global procedure reset_font()
    
    -- reset the fonts to the default BIOS

    sequence  inReg, outReg             -- registers
    inReg = repeat( 0, 10 )             -- initialize register list
    inReg[REG_AX] = #3                  -- BIOS reset the video display to 80
                                        -- cols x 25 lines
    outReg = dos_interrupt( #10, inReg )  -- bios call

end procedure
