-- file name: amort.ex
-- usage: ex amort.ex
-- date: 02/05/99
-- Kenneth Rhodes
-- krhodes@cybersouth.com
--
--                        -------------------------------------------
--                        | compute table of payments on a mortgage |
--                        ------------------------------------------- 
--
--  adapted from mortg.c, "Learning to Program in C", Thomas Plum, p. 3-44
--
--
--  This program has a minor rounding point error. 
--  Any "fix" suggested will be appreciated.
--
--  Also, there are several rough "kludges", most blatant in the wordwrap and 
--  number input and display routines.
--
--
--  I cannot understate my appreciation to the listserver subscribers
--  who have with generosity and tolerance responded to my banal inquiries.
--
--  Special thanks to David Cuny, Jiri Babor, Falkon, Robert Pilkington,
--  Warren Evans....
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
include graphics.e
include get.e
include amort.e  -- includes Jiri Babor's wordwrap.e, slightly modified. THANKS!
include fonter.e  -- David Cuny, THANKS!

object intrmo       -- monthly interest
object intryr       -- annual interest
object prn          -- principal
object prnpmt_subt  -- subtotal amount paid to principal
object bal          -- principal balance remaining
object pmt          -- monthly payment
object prnpmt       -- payment allocated to principal
object intrpmt      -- payment allocated to interest
object intrpmt_subt -- subtotal amount paid to interest
object npmts        -- number of payments
object nyrs         -- number of years
object totalpmt     -- interest payment + principal payment (monthly)
integer lm, rm      -- left margin, right margin
integer line, column    --  screen placement coordinants    
integer totals_line, totals_column -- placement coordinants for totals
integer pause   -- variable to receive return value of wait_key()
integer x       -- receives copy of loop index value "i" 
		-- to compute subtotal of total payments

sequence instruct_start, instruct_end, table_start, table_end

lm = 5
rm = 70

totals_line = 0
totals_column = 0


 load_font_file( "hand.fnt" )-- Thanks to David Cuny's, Fonter.e
 
 -------------------------------------------------------------------------------
 --------------------------------< procedures >---------------------------------
 -------------------------------------------------------------------------------
 procedure exit_check() -- ends program if "0" is entered for prn, intryr, nyrs
  if prn = 0 or intryr= 0 or nyrs = 0 then
   quit() -- mortgage.e
  end if
 end procedure

procedure show_balance()
    -- show current balance
    text_color(RED)
    position(10,lm)
    printf(1,"%8s %10s %10s %10s %10.2f",
	    {" ", " ", " ", " ",bal})                                 
    
end procedure

procedure wait()    -- pause screen display 
	cursor(NO_CURSOR)
	CenterDisplay(25,lm,rm,MAGENTA,"~~Hit any key to continue or '0' to quit~~")
	pause = wait_key()
	cursor(UNDERLINE_CURSOR)
	if pause = 48 then      -- exit program if 0 is pressed
	    quit() -- mortgage.e
	    else
	    show_balance()
	end if
end procedure

procedure sub_total()
	position(line+1,column)
	text_color(BROWN)
	printf(1,"%8s %10.2f %10.2f %10.2f %10.2s", 
		{"", x*totalpmt, intrpmt_subt, prnpmt_subt,""})
	table_end = get_position()
	position(line+1,column)
	puts(1,"Total ==>")
	wait()
end procedure

procedure instruct()    -- only used once, declared as procedure
			-- to make amort() more readable
    sequence instructions
    -- i. e. "right margin" for instructions -length of wordwrap
    integer right_margin
    right_margin = 65
    set_right_margin(right_margin)-- for wordwrap.e, Jiri Babor
    instructions =  "A simple amortization program... you enter numeric "&
		    "values for principal, interest, and number of years. "&
		    "The program displays an amortization schedule in "&
		    "increments of 12 payments.  The cursor will return to "&
		    "the input position if an invalid entry is made.  You "&
		    "can quit the program at any time by entering '0'."
    
    lm = 10             -- indent instructions
    position(10,lm)     ----------------------
    instruct_start = {10,lm}
    text_color(BLUE)
    puts(1, wordwrap(instructions))  -- Thanks to Jiri Babor!!
    instruct_end = get_position()
    instruct_end = {instruct_end[1], right_margin}
    lm=5    -- reset margin
end procedure

procedure get_data()    -- likewise, declared as procedure to 
			-- make amort() more readable

sequence input_start
prn=1                   -- dummy values for exit check
intryr=2                -- and index of input positions
nyrs=3                  -------------------------------
    
    input_start = {             -- sequence for input position
		    {0,0},      -- of values prn,
		    {0,0},      -- intryr, and
		    {0,0}       -- nyrs.
		  }
			-- Prompt_For_Nput(), amort.e
    input_start[prn] = Prompt_For_Nput({3,lm}, CYAN, "Enter principal (e.g. 82500.00):            ")
    input_start[intryr] = Prompt_For_Nput({4,lm}, CYAN, "Enter annual interest rate (e.g. 16.25):    ")
    input_start[nyrs] = Prompt_For_Nput({5,lm},CYAN, "Enter number of years:                      ")
	    --Nput(), amort.e
    prn = Nput({BRIGHT_CYAN,'$'},input_start[prn],{0, 10000000},80)
	exit_check()
    intryr = Nput({BRIGHT_CYAN,'%'},input_start[intryr],{0,75},80)
	exit_check()
    nyrs = Nput({BRIGHT_CYAN,'#'},input_start[nyrs],{0,50},80)
	exit_check()
    
end procedure

-------------------------------------------------------------------------------
---------------------------< Main Procedure amort.ex >-------------------------
-------------------------------------------------------------------------------

procedure amort()
    
    clear_screen()
    
    instruct()
    
    CenterDisplay(1,lm,rm,RED,"COMPUTE TABLE OF PAYMENTS ON MORTGAGE")
    
    get_data()
    
    bal =  prn
    intryr = intryr/100
    intrmo = intryr/12
    npmts = nyrs*12
    intrpmt_subt = 0
    prnpmt_subt = 0

    wipe({instruct_start[1], instruct_start[2]},
	 {instruct_end[1],instruct_end[2]})

    text_color(GREEN)
    position(7,lm)   -- displays variable values given by user
    printf(1,"principal = $%8.2f ", prn)
    printf(1," interest rate = %2.2f ", intryr)
    printf(1," # payments = %2.2f ", npmts)
    -- column headings
    position(9,lm)
    text_color(BROWN)
    printf(1,"%8s %10s %10s %10s %10s\n", 
	    {"payment", "monthly", "interest", "principal", "balance"})
    
    position(10,lm)
    show_balance()
    -- begin computation and display of amortization schedule
    pmt = bal*(intrmo/(1.0-power(1.0+intrmo,-npmts)))
    
    line = 10       -- reference coordinants for loop to display table
    column = lm      -- after wait_key() if return ! = 0
    table_start = {line+1,column}
    -- display schedule in increments of 12
    -- schedule starts display at position(11,1) and ends at position(23,80)
    for i = 1 to npmts  do
	x = i   -- variable used to compute subtotals 
	intrpmt = bal * intrmo
	if i<npmts then
	    prnpmt = pmt -intrpmt
	else
	    prnpmt = bal
	end if
	bal = bal-prnpmt
	totalpmt = intrpmt + prnpmt
	line += 1
	
	if line=23 then -- only show 12 payments per screen     

	    wipe({table_start[1],table_start[2]},{table_end[1],table_end[2]}) --from mortgage.e
	    
	    line=11 -- reset to 1st line of table display
	end if              --  set to show 12 payments per screen
	
	position(line,column)
	text_color(CYAN)
	printf(1,"%8d %10.2f %10.2f %10.2f %10.2f\n", 
		{i, totalpmt, intrpmt,prnpmt, bal}) 
	intrpmt_subt += intrpmt     -- cumulative interest paid.
	prnpmt_subt +=  prnpmt      --     "     principal paid.
	if remainder(i,12) = 0 and i< npmts then    -- pause and display
	     sub_total()                            -- subtotal after display
	end if                                      -- 12 payments
    end for
    sub_total()
    amort()   
end procedure   
------------------------------------------------------------------------------
------------------------------< End amort.ex >--------------------------------
------------------------------------------------------------------------------
amort()

